﻿local addon, ns = ... 
local C, F, G, L = unpack(ns)
if not C.Bags then return end

local format = string.format
local CreateFrame = CreateFrame
local GetContainerNumFreeSlots, GetContainerNumSlots = GetContainerNumFreeSlots, GetContainerNumSlots

--=================================================--
---------------    [[ Elements ]]     ---------------
--=================================================--

--[[ Create elements ]]--
local Stat = CreateFrame("Frame", G.addon.."Bags", UIParent)
	Stat:SetHitRectInsets(-5, -5, -10, -10)
	Stat:SetFrameStrata("BACKGROUND")

--[[ Create text ]]--
local Text  = Stat:CreateFontString(nil, "OVERLAY")
	Text:SetFont(G.Fonts, G.FontSize, G.FontFlag)
	Text:SetPoint(unpack(C.BagsPoint))
	Stat:SetAllPoints(Text)

--==================================================--
---------------    [[ Bag slots ]]     ---------------
--==================================================--

local function getBagSlots()
	local free, total, used = 0, 0, 0
	
	for i = 0, NUM_BAG_SLOTS do
		free, total = free + GetContainerNumFreeSlots(i), total + GetContainerNumSlots(i)
	end
	used = total - free
	
	return free, total, used
end

--================================================--
---------------    [[ Updates ]]     ---------------
--================================================--

--[[ Data text update ]]--
local function OnEvent(self)
	if diminfo.AutoSell == nil then
		diminfo.AutoSell = true
	end

	local free, total = getBagSlots()
	Text:SetText(C.ClassColor and F.Hex(G.Ccolors)..BAGSLOT.." |r"..free.."/"..total or BAGSLOT.." "..free.."/"..total)
	self:SetAllPoints(Text)
end

--[[ Tooltip update ]]--
local function OnEnter(self)
	local free, total, used = getBagSlots()
	local money = GetMoney()
	
	-- title
	GameTooltip:SetOwner(self, C.StickTop and "ANCHOR_BOTTOM" or "ANCHOR_TOP", 0, C.StickTop and -10 or 10)
	GameTooltip:ClearLines()
	GameTooltip:AddDoubleLine(BAGSLOT, free.."/"..total, 0, .6, 1, 0, .6, 1)
	GameTooltip:AddLine(" ")
	
	-- bag slot
	GameTooltip:AddLine(G.OptionColor..BAGSLOT)
	GameTooltip:AddDoubleLine(USE, used, 1, 1, 1, 1, 1, 1)
	GameTooltip:AddDoubleLine(MONEY, GetMoneyString(money), 1, 1, 1, 1, 1, 1)
	
	-- options
	GameTooltip:AddDoubleLine(" ", G.Line)
	GameTooltip:AddDoubleLine(" ", G.OptionColor..BAGSLOT..G.LeftButton)
	GameTooltip:AddDoubleLine(" ", G.OptionColor..L.AutoSell..(diminfo.AutoSell and G.Enable or G.Disable)..G.RightButton, 1, 1, 1, .4, .78, 1)
	
	GameTooltip:Show()
end

--================================================--
---------------    [[ Scripts ]]     ---------------
--================================================--
	
	--[[ Tooltip ]]--
	Stat:SetScript("OnEnter", OnEnter)
	Stat:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	
	--[[ Data text ]]--
	Stat:RegisterEvent("PLAYER_LOGIN")
	Stat:RegisterEvent("BAG_UPDATE")
	Stat:SetScript("OnEvent", OnEvent)
	
	--[[ Options ]]--
	Stat:SetScript("OnMouseDown", function(self, button)
		if button == "RightButton" then
			diminfo.AutoSell = not diminfo.AutoSell
			OnEnter(self)
		elseif button == "LeftButton" then
			ToggleAllBags()
		else
			return
		end
	end)

--=======================================================--
---------------    [[ Auto sell gray ]]     ---------------
--=======================================================--

local sellGray = CreateFrame("Frame")
	sellGray:SetScript("OnEvent", function()
		if diminfo.AutoSell == true then
			local c = 0
			
			for bag = 0, 4 do
				for slot = 1, GetContainerNumSlots(bag) do
					local link = GetContainerItemLink(bag, slot)
					
					if link and (select(11, GetItemInfo(link)) ~= nil) and (select(2, GetContainerItemInfo(bag, slot)) ~= nil) then
						local price = select(11, GetItemInfo(link)) * select(2, GetContainerItemInfo(bag, slot))
						
						if select(3, GetItemInfo(link)) == 0 and price > 0 then
							UseContainerItem(bag, slot)
							PickupMerchantItem()
							c = c + price
						end
					end
				end
			end
			
			if c > 0 then
				print(format("|cff99CCFF"..L.TrashSold.."|r%s", GetMoneyString(c)))
			end
		end
	end)
	sellGray:RegisterEvent("MERCHANT_SHOW")
